@@ -344,7 +344,7 @@ bool Cribbage::haveGO(const Player &player)
 void Cribbage::scoreInPlay(Player &player)
 {
     CardSequence testSequence;
-    CardProperties cardProperties(m_playSequence);
+    CardProperties playSequenceProperties(m_playSequence);
     vector<CardSequence> sequencesFound;
     int points=0, size=0;
 
@@ -360,25 +360,76 @@ void Cribbage::scoreInPlay(Player &player)
         points+=CribbageRules::THIRTY_ONE;
     }
 
+
+    //Are there any runs, by rules only regular runs (2 3 4) are counted
+    if (m_playSequence.size() >= 3)
+    {
+        CardSequence runSequence;
+        CardProperties runProperties;
+
+        // Find Possible run
+        for (int index=m_playSequence.size() - 3; index >= 0; --index)
+        {
+            testSequence.clear();
+            // Build test sequence
+            for (int cardIndex=m_playSequence.size() - 1; cardIndex >= index; --cardIndex)
+                testSequence.addCard(m_playSequence[cardIndex]);
+            runProperties.setSequence(testSequence);
+            sequencesFound=runProperties.runs();
+            for (int i=0, size=sequencesFound.size(); i < size; ++i)
+                if (sequencesFound[i].size() == (m_playSequence.size() - index))
+                    runSequence = sequencesFound[i];
+        }
+        if (! runSequence.isEmpty())
+        {
+            m_pInterface->displayMessage(m_players.currentPlayer(), QString("%1 for a run.").arg(runSequence.size() * CribbageRules::RUN));
+            points += (runSequence.size() * CribbageRules::RUN);
+        }
+    }
+
     //Are there a pairs
-    testSequence.clear();
-    sequencesFound=cardProperties.pairs();
+    sequencesFound=playSequenceProperties.pairs();
     m_rules.createPairTypes(sequencesFound);
     if ((!sequencesFound.empty()) && (!sequencesFound.front().isEmpty()))
     {
-        for (int seqFndN=0; seqFndN < static_cast<int>(sequencesFound.size()); ++seqFndN)
+        for (int sequenceFoundIndex=0, size=sequencesFound.size(); sequenceFoundIndex < size; ++sequenceFoundIndex)
         {
             // Add as many cards to our testSequence as we have in the sequence found.
             testSequence.clear();
-            for (int i=0, j=m_playSequence.size() - 1; (i < sequencesFound[seqFndN].size()) && (j >= 0); ++i, j--)
-                testSequence.addCard(m_playSequence[j]);
-            sort(testSequence.begin(), testSequence.end());
-            sort(sequencesFound[seqFndN].begin(), sequencesFound[seqFndN].end());
-            testSequence.sortBySuit();
-            sequencesFound[seqFndN].sortBySuit();
-            if (testSequence == sequencesFound[seqFndN])
+            for (int sequenceFoundSize=0, playSequenceIndex=m_playSequence.size() - 1; (sequenceFoundSize < sequencesFound[sequenceFoundIndex].size()) && (playSequenceIndex >= 0); ++sequenceFoundSize, playSequenceIndex--)
+                testSequence.addCard(m_playSequence[playSequenceIndex]);
+
+            // Remove any cards that can bork our run test
+            CardProperties testProp(testSequence);
+            vector<CardSequence> testSequences=m_rules.createPairTypes(testProp.pairs()); // Remove non-pair cards
+            if (! testSequences.empty())
             {
-                switch (m_rules.pairType(sequencesFound[seqFndN]))
+                for (int index=0, size=testSequences.size(); index < size; ++index)
+                    if (testSequences[index].hasCard(m_playSequence.back()))
+                    {
+                        testSequence=testSequences[index];
+                        break;
+                    }
+            }
+            else
+                testSequence.clear();
+
+            for (int index=0, endCardIndex=m_playSequence.size() - testSequence.size(), size=sequencesFound[sequenceFoundIndex].size() - 1; index < endCardIndex; ++index)
+            {
+                // If sequencesFound[sequenceFoundIndex] is before endCard remove it
+                if (sequencesFound[sequenceFoundIndex].hasCard(m_playSequence[index]))
+                    sequencesFound[sequenceFoundIndex].removeCard(m_playSequence[index]);
+            }
+
+            testSequence.sortBySuit();
+            sequencesFound[sequenceFoundIndex].sortBySuit();
+            sort(testSequence.begin(), testSequence.end());
+            sort(sequencesFound[sequenceFoundIndex].begin(), sequencesFound[sequenceFoundIndex].end());
+
+            // Test our pair sequence
+            if (testSequence == sequencesFound[sequenceFoundIndex])
+            {
+                switch (m_rules.pairType(sequencesFound[sequenceFoundIndex]))
                 {
                 case CribbageRules::PAIR:
                     m_pInterface->displayMessage(m_players.currentPlayer(), QString("%1 for a pair.").arg(CribbageRules::PAIR));
@@ -399,24 +450,6 @@ void Cribbage::scoreInPlay(Player &player)
         }
     }
 
-    //Are there any runs
-    sequencesFound=cardProperties.runs();
-    size=sequencesFound.size();
-    for (int seqFound=0; seqFound < size; seqFound++)
-    {
-        testSequence.clear();
-        // Add as many cards to our testSequence as we have in the sequence found.
-        for (int i=0, j=m_playSequence.size() - 1; (i < sequencesFound[seqFound].size()) && (j >= 0); ++i, j--)
-            testSequence.addCard(m_playSequence[j]);
-        sort(testSequence.begin(), testSequence.end());
-        sort(sequencesFound[seqFound].begin(), sequencesFound[seqFound].end());
-        if (testSequence == sequencesFound[seqFound])
-        {
-            m_pInterface->displayMessage(m_players.currentPlayer(), QString("%1 for a run.").arg(sequencesFound[seqFound].size()));
-            points+=sequencesFound[seqFound].size();
-        }
-    }
-
     // Did the player earn any points?
     if (points > 0)
         player.addPoint(points);
@@ -691,6 +724,9 @@ int Cribbage::removePairPoints(const CardSequence &run)
                 // Make sure it's a pair in the card summary
                 if (m_cardSummary[cardSummaryIndex][0].rank() != m_cardSummary[cardSummaryIndex][1].rank())
                     continue;
+                // Make sure to only remove the proil when it's 5 5 5
+                else if ((! CardProperties(m_cardSummary[cardSummaryIndex]).summations(15, m_rules).empty()) && (! m_messageSummary[cardSummaryIndex].contains("proil")))
+                    continue;
                 else
                     if (m_cardSummary[cardSummaryIndex].size() == pair.size())
                         foundPair=true;
